@@ -30,6 +30,7 @@ import org.apache.flume.sink.hdfs.HDFSEventSink.WriterCallback;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
+import org.apache.hadoop.hdfs.DistributedFileSystem;
 import org.apache.hadoop.io.SequenceFile.CompressionType;
 import org.apache.hadoop.io.compress.CompressionCodec;
 import org.slf4j.Logger;
@@ -106,7 +107,6 @@ class BucketWriter {
 
   private boolean mockFsInjected = false;
 
-  private Clock clock = new SystemClock();
   private final long retryInterval;
   private final int maxRenameTries;
 
@@ -124,6 +124,26 @@ class BucketWriter {
       String onCloseCallbackPath, long callTimeout,
       ExecutorService callTimeoutPool, long retryInterval,
       int maxCloseTries) {
+    this(rollInterval, rollSize, rollCount, batchSize,
+            context, filePath, fileName, inUsePrefix,
+            inUseSuffix, fileSuffix, codeC,
+            compType, writer,
+            timedRollerPool, proxyUser,
+            sinkCounter, idleTimeout, onCloseCallback,
+            onCloseCallbackPath, callTimeout,
+            callTimeoutPool, retryInterval,
+            maxCloseTries, new SystemClock());
+  }
+
+  BucketWriter(long rollInterval, long rollSize, long rollCount, long batchSize,
+           Context context, String filePath, String fileName, String inUsePrefix,
+           String inUseSuffix, String fileSuffix, CompressionCodec codeC,
+           CompressionType compType, HDFSWriter writer,
+           ScheduledExecutorService timedRollerPool, PrivilegedExecutor proxyUser,
+           SinkCounter sinkCounter, int idleTimeout, WriterCallback onCloseCallback,
+           String onCloseCallbackPath, long callTimeout,
+           ExecutorService callTimeoutPool, long retryInterval,
+           int maxCloseTries, Clock clock) {
     this.rollInterval = rollInterval;
     this.rollSize = rollSize;
     this.rollCount = rollCount;
@@ -179,10 +199,10 @@ class BucketWriter {
       return fileSystem.getClass().getMethod("isFileClosed",
         Path.class);
     } catch (Exception e) {
-      LOG.warn("isFileClosed is not available in the " +
-          "version of HDFS being used. Flume will not " +
-          "attempt to close files if the close fails on " +
-          "the first attempt",e);
+      LOG.info("isFileClosed() is not available in the version of the " +
+               "distributed filesystem being used. " +
+               "Flume will not attempt to re-close files if the close fails " +
+               "on the first attempt");
       return null;
     }
   }
@@ -339,6 +359,22 @@ class BucketWriter {
     };
   }
 
+  /**
+   * Tries to start the lease recovery process for the current bucketPath
+   * if the fileSystem is DistributedFileSystem.
+   * Catches and logs the IOException.
+   */
+  private synchronized void recoverLease() {
+    if (bucketPath != null && fileSystem instanceof DistributedFileSystem) {
+      try {
+        LOG.debug("Starting lease recovery for {}", bucketPath);
+        ((DistributedFileSystem) fileSystem).recoverLease(new Path(bucketPath));
+      } catch (IOException ex) {
+        LOG.warn("Lease recovery failed for {}", bucketPath, ex);
+      }
+    }
+  }
+
   /**
    * Close the file handle and rename the temp file to the permanent filename.
    * Safe to call multiple times. Logs HDFSWriter.close() exceptions.
@@ -353,7 +389,7 @@ class BucketWriter {
     } catch (IOException e) {
       LOG.warn("pre-close flush failed", e);
     }
-    boolean failedToClose = false;
+
     LOG.info("Closing {}", bucketPath);
     CallRunner<Void> closeCallRunner = createCloseCallRunner();
     if (isOpen) {
@@ -364,7 +400,8 @@ class BucketWriter {
         LOG.warn("failed to close() HDFSWriter for file (" + bucketPath +
                  "). Exception follows.", e);
         sinkCounter.incrementConnectionFailedCount();
-        failedToClose = true;
+        // starting lease recovery process, see FLUME-3080
+        recoverLease();
       }
       isOpen = false;
     } else {
@@ -634,10 +671,6 @@ class BucketWriter {
     return (batchCounter == 0);
   }
 
-  void setClock(Clock clock) {
-    this.clock = clock;
-  }
-
   /**
    * This method if the current thread has been interrupted and throws an
    * exception.
