@@ -28,6 +28,7 @@ import javax.jms.JMSException;
 import javax.naming.InitialContext;
 import javax.naming.NamingException;
 
+import org.apache.commons.lang.StringUtils;
 import org.apache.flume.ChannelException;
 import org.apache.flume.Context;
 import org.apache.flume.Event;
@@ -53,7 +54,6 @@ public class JMSSource extends AbstractPollableSource {
   private static final Logger logger = LoggerFactory.getLogger(JMSSource.class);
 
   // setup by constructor
-  private final JMSMessageConsumerFactory consumerFactory;
   private final InitialContextFactory initialContextFactory;
 
   // setup by configuration
@@ -72,44 +72,43 @@ public class JMSSource extends AbstractPollableSource {
   private SourceCounter sourceCounter;
   private int errorThreshold;
   private long pollTimeout;
+  private Optional<String> clientId;
+  private boolean createDurableSubscription;
+  private String durableSubscriptionName;
 
   private int jmsExceptionCounter;
   private InitialContext initialContext;
 
-
   public JMSSource() {
-    this(new JMSMessageConsumerFactory(), new InitialContextFactory());
+    this(new InitialContextFactory());
   }
-  @VisibleForTesting
-  public JMSSource(JMSMessageConsumerFactory consumerFactory, InitialContextFactory initialContextFactory) {
-    super();
-    this.consumerFactory = consumerFactory;
-    this.initialContextFactory = initialContextFactory;
 
+  public JMSSource(InitialContextFactory initialContextFactory) {
+    super();
+    this.initialContextFactory = initialContextFactory;
   }
 
   @Override
   protected void doConfigure(Context context) throws FlumeException {
     sourceCounter = new SourceCounter(getName());
 
-    initialContextFactoryName = context.getString(JMSSourceConfiguration.
-        INITIAL_CONTEXT_FACTORY, "").trim();
+    initialContextFactoryName = context.getString(
+        JMSSourceConfiguration.INITIAL_CONTEXT_FACTORY, "").trim();
 
-    providerUrl = context.getString(JMSSourceConfiguration.PROVIDER_URL, "")
-        .trim();
+    providerUrl = context.getString(JMSSourceConfiguration.PROVIDER_URL, "").trim();
 
-    destinationName = context.getString(JMSSourceConfiguration.
-        DESTINATION_NAME, "").trim();
+    destinationName = context.getString(JMSSourceConfiguration.DESTINATION_NAME, "").trim();
 
-    String destinationTypeName = context.getString(JMSSourceConfiguration.
-        DESTINATION_TYPE, "").trim().toUpperCase(Locale.ENGLISH);
+    String destinationTypeName = context.getString(
+        JMSSourceConfiguration.DESTINATION_TYPE, "").trim().toUpperCase(Locale.ENGLISH);
 
-    String destinationLocatorName = context.getString(JMSSourceConfiguration.
-        DESTINATION_LOCATOR, JMSSourceConfiguration.DESTINATION_LOCATOR_DEFAULT)
-      .trim().toUpperCase(Locale.ENGLISH);
+    String destinationLocatorName = context.getString(
+        JMSSourceConfiguration.DESTINATION_LOCATOR,
+        JMSSourceConfiguration.DESTINATION_LOCATOR_DEFAULT)
+        .trim().toUpperCase(Locale.ENGLISH);
 
-    messageSelector = context.getString(JMSSourceConfiguration.
-        MESSAGE_SELECTOR, "").trim();
+    messageSelector = context.getString(
+        JMSSourceConfiguration.MESSAGE_SELECTOR, "").trim();
 
     batchSize = context.getInteger(JMSSourceConfiguration.BATCH_SIZE,
         JMSSourceConfiguration.BATCH_SIZE_DEFAULT);
@@ -117,16 +116,23 @@ public class JMSSource extends AbstractPollableSource {
     errorThreshold = context.getInteger(JMSSourceConfiguration.ERROR_THRESHOLD,
         JMSSourceConfiguration.ERROR_THRESHOLD_DEFAULT);
 
-    userName = Optional.fromNullable(context.getString(JMSSourceConfiguration.
-        USERNAME));
+    userName = Optional.fromNullable(context.getString(JMSSourceConfiguration.USERNAME));
 
     pollTimeout = context.getLong(JMSSourceConfiguration.POLL_TIMEOUT,
         JMSSourceConfiguration.POLL_TIMEOUT_DEFAULT);
 
-    String passwordFile = context.getString(JMSSourceConfiguration.
-        PASSWORD_FILE, "").trim();
+    clientId = Optional.fromNullable(context.getString(JMSSourceConfiguration.CLIENT_ID));
 
-    if(passwordFile.isEmpty()) {
+    createDurableSubscription = context.getBoolean(
+        JMSSourceConfiguration.CREATE_DURABLE_SUBSCRIPTION, 
+        JMSSourceConfiguration.DEFAULT_CREATE_DURABLE_SUBSCRIPTION);
+    durableSubscriptionName = context.getString(
+        JMSSourceConfiguration.DURABLE_SUBSCRIPTION_NAME, 
+        JMSSourceConfiguration.DEFAULT_DURABLE_SUBSCRIPTION_NAME);
+
+    String passwordFile = context.getString(JMSSourceConfiguration.PASSWORD_FILE, "").trim();
+
+    if (passwordFile.isEmpty()) {
       password = Optional.of("");
     } else {
       try {
@@ -140,45 +146,38 @@ public class JMSSource extends AbstractPollableSource {
 
     String converterClassName = context.getString(
         JMSSourceConfiguration.CONVERTER_TYPE,
-        JMSSourceConfiguration.CONVERTER_TYPE_DEFAULT)
-        .trim();
-    if(JMSSourceConfiguration.CONVERTER_TYPE_DEFAULT.
-        equalsIgnoreCase(converterClassName)) {
+        JMSSourceConfiguration.CONVERTER_TYPE_DEFAULT).trim();
+    if (JMSSourceConfiguration.CONVERTER_TYPE_DEFAULT.equalsIgnoreCase(converterClassName)) {
       converterClassName = DefaultJMSMessageConverter.Builder.class.getName();
     }
-    Context converterContext = new Context(context.
-        getSubProperties(JMSSourceConfiguration.CONVERTER + "."));
+    Context converterContext = new Context(context.getSubProperties(
+        JMSSourceConfiguration.CONVERTER + "."));
     try {
       @SuppressWarnings("rawtypes")
       Class clazz = Class.forName(converterClassName);
       boolean isBuilder = JMSMessageConverter.Builder.class
           .isAssignableFrom(clazz);
-      if(isBuilder) {
-        JMSMessageConverter.Builder builder = (JMSMessageConverter.Builder)
-            clazz.newInstance();
+      if (isBuilder) {
+        JMSMessageConverter.Builder builder = (JMSMessageConverter.Builder)clazz.newInstance();
         converter = builder.build(converterContext);
       } else {
-        Preconditions.checkState(JMSMessageConverter.class.
-            isAssignableFrom(clazz), String.
-            format("Class %s is not a subclass of JMSMessageConverter",
-                clazz.getName()));
+        Preconditions.checkState(JMSMessageConverter.class.isAssignableFrom(clazz),
+            String.format("Class %s is not a subclass of JMSMessageConverter", clazz.getName()));
         converter = (JMSMessageConverter)clazz.newInstance();
-        boolean configured = Configurables.configure(converter,
-            converterContext);
-        if(logger.isDebugEnabled()) {
-          logger.debug(String.
-              format("Attempted configuration of %s, result = %s",
-                  converterClassName, String.valueOf(configured)));
+        boolean configured = Configurables.configure(converter, converterContext);
+        if (logger.isDebugEnabled()) {
+          logger.debug(String.format("Attempted configuration of %s, result = %s",
+                                     converterClassName, String.valueOf(configured)));
         }
       }
-    } catch(Exception e) {
+    } catch (Exception e) {
       throw new FlumeException(String.format(
           "Unable to create instance of converter %s", converterClassName), e);
     }
 
-    String connectionFactoryName = context.getString(JMSSourceConfiguration.
-        CONNECTION_FACTORY, JMSSourceConfiguration.CONNECTION_FACTORY_DEFAULT)
-        .trim();
+    String connectionFactoryName = context.getString(
+        JMSSourceConfiguration.CONNECTION_FACTORY,
+        JMSSourceConfiguration.CONNECTION_FACTORY_DEFAULT).trim();
 
     assertNotEmpty(initialContextFactoryName, String.format(
         "Initial Context Factory is empty. This is specified by %s",
@@ -203,6 +202,30 @@ public class JMSSource extends AbstractPollableSource {
           "invalid.", destinationTypeName), e);
     }
 
+    if (createDurableSubscription) {
+      if (JMSDestinationType.TOPIC != destinationType) {
+        throw new FlumeException(String.format(
+            "Only Destination type '%s' supports durable subscriptions.",
+            JMSDestinationType.TOPIC.toString()));
+      }
+      if (!clientId.isPresent()) {
+        throw new FlumeException(String.format(
+            "You have to specify '%s' when using durable subscriptions.",
+            JMSSourceConfiguration.CLIENT_ID));
+      }
+      if (StringUtils.isEmpty(durableSubscriptionName)) {
+        throw new FlumeException(String.format("If '%s' is set to true, '%s' has to be specified.",
+            JMSSourceConfiguration.CREATE_DURABLE_SUBSCRIPTION,
+            JMSSourceConfiguration.DURABLE_SUBSCRIPTION_NAME));
+      }
+    } else if (!StringUtils.isEmpty(durableSubscriptionName)) {
+      logger.warn(String.format("'%s' is set, but '%s' is false."
+          + "If you want to create a durable subscription, set %s to true.",
+          JMSSourceConfiguration.DURABLE_SUBSCRIPTION_NAME,
+          JMSSourceConfiguration.CREATE_DURABLE_SUBSCRIPTION,
+          JMSSourceConfiguration.CREATE_DURABLE_SUBSCRIPTION));
+    }
+
     try {
       destinationLocator = JMSDestinationLocator.valueOf(destinationLocatorName);
     } catch (IllegalArgumentException e) {
@@ -210,8 +233,7 @@ public class JMSSource extends AbstractPollableSource {
           "invalid.", destinationLocatorName), e);
     }
 
-    Preconditions.checkArgument(batchSize > 0, "Batch size must be greater " +
-        "than 0");
+    Preconditions.checkArgument(batchSize > 0, "Batch size must be greater than 0");
 
     try {
       Properties contextProperties = new Properties();
@@ -220,6 +242,17 @@ public class JMSSource extends AbstractPollableSource {
           initialContextFactoryName);
       contextProperties.setProperty(
           javax.naming.Context.PROVIDER_URL, providerUrl);
+
+      // Provide properties for connecting via JNDI
+      if (this.userName.isPresent()) {
+        contextProperties.setProperty(javax.naming.Context.SECURITY_PRINCIPAL,
+                                      this.userName.get());
+      }
+      if (this.password.isPresent()) {
+        contextProperties.setProperty(javax.naming.Context.SECURITY_CREDENTIALS,
+                                      this.password.get());
+      }
+
       initialContext = initialContextFactory.create(contextProperties);
     } catch (NamingException e) {
       throw new FlumeException(String.format(
@@ -228,28 +261,26 @@ public class JMSSource extends AbstractPollableSource {
     }
 
     try {
-      connectionFactory = (ConnectionFactory) initialContext.
-          lookup(connectionFactoryName);
+      connectionFactory = (ConnectionFactory) initialContext.lookup(connectionFactoryName);
     } catch (NamingException e) {
       throw new FlumeException("Could not lookup ConnectionFactory", e);
     }
   }
 
   private void assertNotEmpty(String arg, String msg) {
-    Preconditions.checkArgument(!arg.isEmpty(),
-        msg);
+    Preconditions.checkArgument(!arg.isEmpty(), msg);
   }
 
   @Override
   protected synchronized Status doProcess() throws EventDeliveryException {
     boolean error = true;
     try {
-      if(consumer == null) {
+      if (consumer == null) {
         consumer = createConsumer();
       }
       List<Event> events = consumer.take();
       int size = events.size();
-      if(size == 0) {
+      if (size == 0) {
         error = false;
         return Status.BACKOFF;
       }
@@ -264,28 +295,28 @@ public class JMSSource extends AbstractPollableSource {
       logger.warn("Error appending event to channel. "
           + "Channel might be full. Consider increasing the channel "
           + "capacity or make sure the sinks perform faster.", channelException);
-    } catch(JMSException jmsException) {
+    } catch (JMSException jmsException) {
       logger.warn("JMSException consuming events", jmsException);
-      if(++jmsExceptionCounter > errorThreshold) {
-        if(consumer != null) {
+      if (++jmsExceptionCounter > errorThreshold) {
+        if (consumer != null) {
           logger.warn("Exceeded JMSException threshold, closing consumer");
           consumer.rollback();
           consumer.close();
           consumer = null;
         }
       }
-    } catch(Throwable throwable) {
+    } catch (Throwable throwable) {
       logger.error("Unexpected error processing events", throwable);
-      if(throwable instanceof Error) {
+      if (throwable instanceof Error) {
         throw (Error) throwable;
       }
     } finally {
-      if(error) {
-        if(consumer != null) {
+      if (error) {
+        if (consumer != null) {
           consumer.rollback();
         }
       } else {
-        if(consumer != null) {
+        if (consumer != null) {
           consumer.commit();
           jmsExceptionCounter = 0;
         }
@@ -293,6 +324,7 @@ public class JMSSource extends AbstractPollableSource {
     }
     return Status.BACKOFF;
   }
+
   @Override
   protected synchronized void doStart() {
     try {
@@ -306,18 +338,20 @@ public class JMSSource extends AbstractPollableSource {
 
   @Override
   protected synchronized void doStop() {
-    if(consumer != null) {
+    if (consumer != null) {
       consumer.close();
       consumer = null;
     }
     sourceCounter.stop();
   }
-  private JMSMessageConsumer createConsumer() throws JMSException {
+
+  @VisibleForTesting
+  JMSMessageConsumer createConsumer() throws JMSException {
     logger.info("Creating new consumer for " + destinationName);
-    JMSMessageConsumer consumer = consumerFactory.create(initialContext,
-    connectionFactory, destinationName, destinationType, destinationLocator,
-    messageSelector, batchSize,
-        pollTimeout, converter, userName, password);
+    JMSMessageConsumer consumer = new JMSMessageConsumer(initialContext,
+        connectionFactory, destinationName, destinationLocator, destinationType,
+        messageSelector, batchSize, pollTimeout, converter, userName, password, clientId,
+        createDurableSubscription, durableSubscriptionName);
     jmsExceptionCounter = 0;
     return consumer;
   }
